# GRAPH_NAME GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)
class GRAPH_NAMEType(DjangoObjectType):
    class Meta:
        name = "GRAPH_NAME"
        description = "Manejo de GRAPH_NAME"
        model = UserApp
        fields = ["last_name",]
# END GRAPH_NAME GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)