# START API API_NAMESerializerDelete (Created by SPRINGLABS_DJANGO)
class API_NAMESerializerDelete(serializers.Serializer):
    def validate(self, validated_data):
        """
            Breve descripción de método validate.

            Descripción detallada de método validate, esta
            descripción detallada puede llevar varias lineas.
            Parámetros:
                param           [Type]          Descripción de param
            Excepciones:
                Excepcion       [Type]          Descripción de excepción
            Retorno:
                retorno        [Type]          Descripción de retorno
        """
        return validated_data

    def destroy(self, instance, validated_data):
        """
            Breve descripción de método destroy.

            Descripción detallada de método destroy, esta
            descripción detallada puede llevar varias lineas.
            Parámetros:
                param           [Type]          Descripción de param
            Excepciones:
                Excepcion       [Type]          Descripción de excepción
            Retorno:
                retorno        [Type]          Descripción de retorno
        """
        return validated_data
# END API API_NAMESerializerDelete (Created by SPRINGLABS_DJANGO)
